
//-----------------------------------------------------------------------------
// Global variables
//-----------------------------------------------------------------------------
float4 MaterialAmbientColor = {0.3, 0.3, 0.3, 1.0};
float4 MaterialDiffuseColor = {0.7, 0.7, 0.7, 0.0};

#define LIGHT_DIR  0.0,10.0,0.0
float3 LightDir = normalize(float3(LIGHT_DIR));
float4 LightAmbient = { 1.0f, 1.0f, 1.0f, 1.0f };    // ambient
float4 LightDiffuse = { 1.0f, 1.0f, 1.0f, 1.0f };    // diffuse

float4x4 mWorld;
float4x4 mWorldViewProjection;

float4 weight;
float coeff_l;
static const float MAP_WIDTH  = 512;
static const float MAP_HEIGHT = 512;
float4 vBias = {2/MAP_WIDTH, 4/MAP_WIDTH, 6/MAP_WIDTH, 8/MAP_WIDTH};
float4 hBias = {2/MAP_HEIGHT,4/MAP_HEIGHT,6/MAP_HEIGHT,8/MAP_HEIGHT};

//-----------------------------------------------------------------------------
// Texture samplers
//-----------------------------------------------------------------------------
texture RenderTargetTexture;
sampler RenderTargetSampler = 
sampler_state
{
    Texture = <RenderTargetTexture>;
    MinFilter = LINEAR;
    MagFilter = LINEAR;

    AddressU = Clamp;
    AddressV = Clamp;
};

texture MeshTexture;
sampler MeshTextureSampler = 
sampler_state
{
    Texture = <MeshTexture>;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    MipFilter = NONE;

    AddressU = Clamp;
    AddressV = Clamp;
};

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
float3 GetDiffuse(float3 normalW )
{
    return MaterialDiffuseColor * LightDiffuse * max(0,dot(normalW, LightDir))
           + MaterialAmbientColor * LightAmbient;
}


//-----------------------------------------------------------------------------
// Vertex shader output structure
//-----------------------------------------------------------------------------
struct VS_OUTPUT
{
    float4 Position : POSITION;
    float2 TextureUV : TEXCOORD0;
    float4 Diffuse : TEXCOORD1;
};
struct PS_OUTPUT
{
    float4 Color : Color0;
};


//-----------------------------------------------------------------------------
// Name: SceneVertexShader
// Type: Vertex shader                                      
// Desc: In addition to standard transform and lighting, it calculates the blur
//       factor of the vertex and outputs this as a texture coord.
//-----------------------------------------------------------------------------
VS_OUTPUT SceneVertexShader( float4 vPos : POSITION, 
                             float3 vNormal : NORMAL,
                             float2 vTexCoord0 : TEXCOORD0 )
{
    VS_OUTPUT Output;
    float3 vViewPosition;
    float3 vWorldNormal;
    float  fBlurFactor;
  
    // tranform vertex position into screen space
    Output.Position = mul(vPos, mWorldViewProjection);
    
    // Compute simple lighting equation
    vWorldNormal = mul(vNormal, (float3x3)mWorld);       
    Output.Diffuse = float4(GetDiffuse(vWorldNormal), 1.0);
    
    // Just copy the texture coordinate through
    Output.TextureUV.xy = vTexCoord0;
    
    return Output;
}


//-----------------------------------------------------------------------------
// Name: ScenePixelShader
// Type: Pixel shader
// Desc: This shader simply outputs the pixel's color 
//-----------------------------------------------------------------------------
PS_OUTPUT ScenePixelShader( VS_OUTPUT In )
{
    PS_OUTPUT Output;

    Output.Color = tex2D(MeshTextureSampler, In.TextureUV.xy) * In.Diffuse;
    Output.Color.a = 1.0f;
    
    return Output;
}


//-----------------------------------------------------------------------------
// Name: TechScene
// Type: Technique                                     
// Desc: Renders the scene's color to the render target and stores the
//       depth information as a blur factor in the alpha channel.
//-----------------------------------------------------------------------------
technique TechScene
{
    pass P0
    {        
        VertexShader = compile vs_1_1 SceneVertexShader();
        PixelShader  = compile ps_2_0 ScenePixelShader();
    }
}




//-----------------------------------------------------------------------------
// Name: RenderGaussX
//-----------------------------------------------------------------------------
float4 RenderGaussX( in float2 OriginalUV : TEXCOORD0 ) : COLOR 
{
     float4 Color;
     
     half4 t00 = tex2D( MeshTextureSampler, OriginalUV );
     half4 t10 = tex2D( MeshTextureSampler, OriginalUV + half2(vBias.x, 0));
     half4 t11 = tex2D( MeshTextureSampler, OriginalUV - half2(vBias.x, 0));
     half4 t20 = tex2D( MeshTextureSampler, OriginalUV + half2(vBias.y, 0));
     half4 t21 = tex2D( MeshTextureSampler, OriginalUV - half2(vBias.y, 0));
     half4 t30 = tex2D( MeshTextureSampler, OriginalUV + half2(vBias.z, 0));
     half4 t31 = tex2D( MeshTextureSampler, OriginalUV - half2(vBias.z, 0));
     half4 t40 = tex2D( MeshTextureSampler, OriginalUV + half2(vBias.w, 0));
     half4 t41 = tex2D( MeshTextureSampler, OriginalUV - half2(vBias.w, 0));
     
     // SƂ̐F̍
     half3 d10 = t10.rgb - t00.rgb;
     half3 d20 = t20.rgb - t00.rgb;
     half3 d30 = t30.rgb - t00.rgb;
     half3 d40 = t40.rgb - t00.rgb;
     half3 d11 = t11.rgb - t00.rgb;
     half3 d21 = t21.rgb - t00.rgb;
     half3 d31 = t31.rgb - t00.rgb;
     half3 d41 = t41.rgb - t00.rgb;

     // SƂ̐F̋̍̂Q
     half4 l0, l1;
     l0.x = dot(d10, d10);
     l0.y = dot(d20, d20);
     l0.z = dot(d30, d30);
     l0.w = dot(d40, d40);
     l1.x = dot(d11, d11);
     l1.y = dot(d21, d21);
     l1.z = dot(d31, d31);
     l1.w = dot(d41, d41);
     
     l0 = weight * exp(coeff_l * l0);
     l1 = weight * exp(coeff_l * l1);

     Color  = t00;
     Color += l0.x * t10 + l1.x * t11;
     Color += l0.y * t20 + l1.y * t21;
     Color += l0.z * t30 + l1.z * t31;
     Color += l0.w * t40 + l1.w * t41;
     
     return Color / 4;// SƐxȂ悤Ȃ̂ŁAKɊƂ
}


technique TechGaussX
{
    pass P0
    {        
        PixelShader = compile ps_2_0 RenderGaussX();
    }
}
//-----------------------------------------------------------------------------
// Name: RenderGaussX
//-----------------------------------------------------------------------------
float4 RenderGaussY( in float2 OriginalUV : TEXCOORD0 ) : COLOR 
{
     float4 Color;
     
     half4 t00 = tex2D( MeshTextureSampler, OriginalUV );
     half4 t10 = tex2D( MeshTextureSampler, OriginalUV + half2(0, hBias.x));
     half4 t11 = tex2D( MeshTextureSampler, OriginalUV - half2(0, hBias.x));
     half4 t20 = tex2D( MeshTextureSampler, OriginalUV + half2(0, hBias.y));
     half4 t21 = tex2D( MeshTextureSampler, OriginalUV - half2(0, hBias.y));
     half4 t30 = tex2D( MeshTextureSampler, OriginalUV + half2(0, hBias.z));
     half4 t31 = tex2D( MeshTextureSampler, OriginalUV - half2(0, hBias.z));
     half4 t40 = tex2D( MeshTextureSampler, OriginalUV + half2(0, hBias.w));
     half4 t41 = tex2D( MeshTextureSampler, OriginalUV - half2(0, hBias.w));
     
     // SƂ̐F̍
     half3 d10 = t10.rgb - t00.rgb;
     half3 d20 = t20.rgb - t00.rgb;
     half3 d30 = t30.rgb - t00.rgb;
     half3 d40 = t40.rgb - t00.rgb;
     half3 d11 = t11.rgb - t00.rgb;
     half3 d21 = t21.rgb - t00.rgb;
     half3 d31 = t31.rgb - t00.rgb;
     half3 d41 = t41.rgb - t00.rgb;

     // SƂ̐F̋̍̂Q
     half4 l0, l1;
     l0.x = dot(d10, d10);
     l0.y = dot(d20, d20);
     l0.z = dot(d30, d30);
     l0.w = dot(d40, d40);
     l1.x = dot(d11, d11);
     l1.y = dot(d21, d21);
     l1.z = dot(d31, d31);
     l1.w = dot(d41, d41);
     
     l0 = weight * exp(coeff_l * l0);
     l1 = weight * exp(coeff_l * l1);

     Color  = t00;
     Color += l0.x * t10 + l1.x * t11;
     Color += l0.y * t20 + l1.y * t21;
     Color += l0.z * t30 + l1.z * t31;
     Color += l0.w * t40 + l1.w * t41;
     
     return Color/Color.a;
}


technique TechGaussY
{
    pass P0
    {        
        PixelShader = compile ps_2_0 RenderGaussY();
    }
}



float4 FinalPS( in float2 OriginalUV : TEXCOORD0 ) : COLOR 
{
	return tex2D(MeshTextureSampler, OriginalUV);
}


//-----------------------------------------------------------------------------
// Name: TechFinal
// Type: Technique
//-----------------------------------------------------------------------------
technique TechFinal
{
    pass P0
    {        
        PixelShader = compile ps_1_1 FinalPS();
        
        ZEnable = false;
        AlphaBlendEnable = false;
    }
}
